#ifndef NPSTAT_COORDANDWEIGHT_HH_
#define NPSTAT_COORDANDWEIGHT_HH_

//======================================================================
// coordAndWeight.hh
//
// This is an internal header which is subject to change without notice.
// Application code should never call the functions declared/defined in
// this header directly.
//
// Author: I. Volobouev
//
// May 2022
//======================================================================

#include <vector>
#include <utility>

namespace npstat {
    namespace Private {
        template<typename Numeric>
        inline std::pair<double,double> coordAndWeight(const Numeric& r)
        {
            return std::pair<double,double>(r, 1.0);
        }

        template<typename Num1, typename Num2>
        inline std::pair<double,double> coordAndWeight(
            const std::pair<Num1, Num2>& r)
        {
            return std::pair<double,double>(r.first, r.second);
        }

        template<typename Numeric>
        struct WeightedCoord
        {
            inline WeightedCoord(const double x, const double /* w */)
                : value(static_cast<Numeric>(x)) {}
            Numeric value;
        };

        template<typename Num1, typename Num2>
        struct WeightedCoord<std::pair<Num1, Num2> >
        {
            inline WeightedCoord(const double x, const double w)
                : value(x, w) {}
            std::pair<Num1, Num2> value;
        };

        template<typename Numeric>
        struct SampleWeight
        {
            inline SampleWeight(const std::vector<Numeric>& vec)
                : value(vec.size()) {}
            double value;
        };

        template<typename Num1, typename Num2>
        struct SampleWeight<std::pair<Num1, Num2> >
        {
            inline SampleWeight(const std::vector<std::pair<Num1, Num2> >& vec)
            {
                long double sum = 0.0L;
                const unsigned long sz = vec.size();
                for (unsigned long i=0; i<sz; ++i)
                    sum += static_cast<long double>(vec[i].second);
                value = sum;
            }
            double value;
        };
    }
}

#endif // NPSTAT_COORDANDWEIGHT_HH_
